@@ -74,6 +74,7 @@ gnutls_bye.restype = c_int
 gnutls_certificate_activation_time_peers = libgnutls.gnutls_certificate_activation_time_peers
 gnutls_certificate_activation_time_peers.argtypes = [gnutls_session_t]
 gnutls_certificate_activation_time_peers.restype = time_t
+gnutls_certificate_activation_time_peers.errmsg = "cannot get certificate activation time"
 
 gnutls_certificate_allocate_credentials = libgnutls.gnutls_certificate_allocate_credentials
 gnutls_certificate_allocate_credentials.argtypes = [POINTER(gnutls_certificate_credentials_t)]
@@ -90,6 +91,7 @@ gnutls_certificate_client_set_retrieve_function.restype = None
 gnutls_certificate_expiration_time_peers = libgnutls.gnutls_certificate_expiration_time_peers
 gnutls_certificate_expiration_time_peers.argtypes = [gnutls_session_t]
 gnutls_certificate_expiration_time_peers.restype = time_t
+gnutls_certificate_expiration_time_peers.errmsg = "cannot get certificate expiration time"
 
 gnutls_certificate_free_ca_names = libgnutls.gnutls_certificate_free_ca_names
 gnutls_certificate_free_ca_names.argtypes = [gnutls_certificate_credentials_t]
@@ -915,62 +917,6 @@ gnutls_sign_callback_set = libgnutls.gnutls_sign_callback_set
 gnutls_sign_callback_set.argtypes = [gnutls_session_t, gnutls_sign_func, c_void_p]
 gnutls_sign_callback_set.restype = None
 
-gnutls_srp_allocate_client_credentials = libgnutls.gnutls_srp_allocate_client_credentials
-gnutls_srp_allocate_client_credentials.argtypes = [POINTER(gnutls_srp_client_credentials_t)]
-gnutls_srp_allocate_client_credentials.restype = c_int
-
-gnutls_srp_allocate_server_credentials = libgnutls.gnutls_srp_allocate_server_credentials
-gnutls_srp_allocate_server_credentials.argtypes = [POINTER(gnutls_srp_server_credentials_t)]
-gnutls_srp_allocate_server_credentials.restype = c_int
-
-gnutls_srp_base64_decode = libgnutls.gnutls_srp_base64_decode
-gnutls_srp_base64_decode.argtypes = [POINTER(gnutls_datum_t), c_char_p, POINTER(size_t)]
-gnutls_srp_base64_decode.restype = c_int
-
-gnutls_srp_base64_decode_alloc = libgnutls.gnutls_srp_base64_decode_alloc
-gnutls_srp_base64_decode_alloc.argtypes = [POINTER(gnutls_datum_t), POINTER(gnutls_datum_t)]
-gnutls_srp_base64_decode_alloc.restype = c_int
-
-gnutls_srp_base64_encode = libgnutls.gnutls_srp_base64_encode
-gnutls_srp_base64_encode.argtypes = [POINTER(gnutls_datum_t), c_char_p, POINTER(size_t)]
-gnutls_srp_base64_encode.restype = c_int
-
-gnutls_srp_base64_encode_alloc = libgnutls.gnutls_srp_base64_encode_alloc
-gnutls_srp_base64_encode_alloc.argtypes = [POINTER(gnutls_datum_t), POINTER(gnutls_datum_t)]
-gnutls_srp_base64_encode_alloc.restype = c_int
-
-gnutls_srp_free_client_credentials = libgnutls.gnutls_srp_free_client_credentials
-gnutls_srp_free_client_credentials.argtypes = [gnutls_srp_client_credentials_t]
-gnutls_srp_free_client_credentials.restype = None
-
-gnutls_srp_free_server_credentials = libgnutls.gnutls_srp_free_server_credentials
-gnutls_srp_free_server_credentials.argtypes = [gnutls_srp_server_credentials_t]
-gnutls_srp_free_server_credentials.restype = None
-
-gnutls_srp_server_get_username = libgnutls.gnutls_srp_server_get_username
-gnutls_srp_server_get_username.argtypes = [gnutls_session_t]
-gnutls_srp_server_get_username.restype = c_char_p
-
-gnutls_srp_set_client_credentials = libgnutls.gnutls_srp_set_client_credentials
-gnutls_srp_set_client_credentials.argtypes = [gnutls_srp_client_credentials_t, c_char_p, c_char_p]
-gnutls_srp_set_client_credentials.restype = c_int
-
-gnutls_srp_set_client_credentials_function = libgnutls.gnutls_srp_set_client_credentials_function
-gnutls_srp_set_client_credentials_function.argtypes = [gnutls_srp_client_credentials_t, gnutls_srp_client_credentials_function]
-gnutls_srp_set_client_credentials_function.restype = None
-
-gnutls_srp_set_server_credentials_file = libgnutls.gnutls_srp_set_server_credentials_file
-gnutls_srp_set_server_credentials_file.argtypes = [gnutls_srp_server_credentials_t, c_char_p, c_char_p]
-gnutls_srp_set_server_credentials_file.restype = c_int
-
-gnutls_srp_set_server_credentials_function = libgnutls.gnutls_srp_set_server_credentials_function
-gnutls_srp_set_server_credentials_function.argtypes = [gnutls_srp_server_credentials_t, gnutls_srp_server_credentials_function]
-gnutls_srp_set_server_credentials_function.restype = None
-
-gnutls_srp_verifier = libgnutls.gnutls_srp_verifier
-gnutls_srp_verifier.argtypes = [c_char_p, c_char_p, POINTER(gnutls_datum_t), POINTER(gnutls_datum_t), POINTER(gnutls_datum_t), POINTER(gnutls_datum_t)]
-gnutls_srp_verifier.restype = c_int
-
 gnutls_strerror = libgnutls.gnutls_strerror
 gnutls_strerror.argtypes = [c_int]
 gnutls_strerror.restype = c_char_p
@@ -1050,6 +996,7 @@ gnutls_x509_crl_get_issuer_dn_by_oid.restype = c_int
 gnutls_x509_crl_get_next_update = libgnutls.gnutls_x509_crl_get_next_update
 gnutls_x509_crl_get_next_update.argtypes = [gnutls_x509_crl_t]
 gnutls_x509_crl_get_next_update.restype = time_t
+gnutls_x509_crl_get_next_update.errmsg = "cannot get CRL's next update time"
 
 gnutls_x509_crl_get_signature = libgnutls.gnutls_x509_crl_get_signature
 gnutls_x509_crl_get_signature.argtypes = [gnutls_x509_crl_t, c_char_p, POINTER(size_t)]
@@ -1062,6 +1009,7 @@ gnutls_x509_crl_get_signature_algorithm.restype = c_int
 gnutls_x509_crl_get_this_update = libgnutls.gnutls_x509_crl_get_this_update
 gnutls_x509_crl_get_this_update.argtypes = [gnutls_x509_crl_t]
 gnutls_x509_crl_get_this_update.restype = time_t
+gnutls_x509_crl_get_this_update.errmsg = "cannot get CRL's issue time"
 
 gnutls_x509_crl_get_version = libgnutls.gnutls_x509_crl_get_version
 gnutls_x509_crl_get_version.argtypes = [gnutls_x509_crl_t]
@@ -1206,6 +1154,7 @@ gnutls_x509_crt_export.restype = c_int
 gnutls_x509_crt_get_activation_time = libgnutls.gnutls_x509_crt_get_activation_time
 gnutls_x509_crt_get_activation_time.argtypes = [gnutls_x509_crt_t]
 gnutls_x509_crt_get_activation_time.restype = time_t
+gnutls_x509_crt_get_activation_time.errmsg = "cannot get X509 certificate activation time"
 
 gnutls_x509_crt_get_authority_key_id = libgnutls.gnutls_x509_crt_get_authority_key_id
 gnutls_x509_crt_get_authority_key_id.argtypes = [gnutls_x509_crt_t, c_void_p, POINTER(size_t), POINTER(c_uint)]
@@ -1238,6 +1187,7 @@ gnutls_x509_crt_get_dn_oid.restype = c_int
 gnutls_x509_crt_get_expiration_time = libgnutls.gnutls_x509_crt_get_expiration_time
 gnutls_x509_crt_get_expiration_time.argtypes = [gnutls_x509_crt_t]
 gnutls_x509_crt_get_expiration_time.restype = time_t
+gnutls_x509_crt_get_expiration_time.errmsg = "cannot get X509 certificate expiration time"
 
 gnutls_x509_crt_get_extension_by_oid = libgnutls.gnutls_x509_crt_get_extension_by_oid
 gnutls_x509_crt_get_extension_by_oid.argtypes = [gnutls_x509_crt_t, c_char_p, c_int, c_void_p, POINTER(size_t), POINTER(c_uint)]
@@ -1627,10 +1577,12 @@ else:
     gnutls_openpgp_crt_get_creation_time = libgnutls.gnutls_openpgp_crt_get_creation_time
     gnutls_openpgp_crt_get_creation_time.argtypes = [gnutls_openpgp_crt_t]
     gnutls_openpgp_crt_get_creation_time.restype = time_t
+    gnutls_openpgp_crt_get_creation_time.errmsg = "cannot get OpenPGP key creation time"
 
     gnutls_openpgp_crt_get_expiration_time = libgnutls.gnutls_openpgp_crt_get_expiration_time
     gnutls_openpgp_crt_get_expiration_time.argtypes = [gnutls_openpgp_crt_t]
     gnutls_openpgp_crt_get_expiration_time.restype = time_t
+    gnutls_openpgp_crt_get_expiration_time.errmsg = "cannot get OpenPGP key expiration time"
 
     gnutls_openpgp_crt_get_fingerprint = libgnutls.gnutls_openpgp_crt_get_fingerprint
     gnutls_openpgp_crt_get_fingerprint.argtypes = [gnutls_openpgp_crt_t, c_void_p, POINTER(size_t)]
@@ -1675,10 +1627,12 @@ else:
     gnutls_openpgp_crt_get_subkey_creation_time = libgnutls.gnutls_openpgp_crt_get_subkey_creation_time
     gnutls_openpgp_crt_get_subkey_creation_time.argtypes = [gnutls_openpgp_crt_t, c_uint]
     gnutls_openpgp_crt_get_subkey_creation_time.restype = time_t
+    gnutls_openpgp_crt_get_subkey_creation_time.errmsg = "cannot get OpenPGP subkey creation time"
 
     gnutls_openpgp_crt_get_subkey_expiration_time = libgnutls.gnutls_openpgp_crt_get_subkey_expiration_time
     gnutls_openpgp_crt_get_subkey_expiration_time.argtypes = [gnutls_openpgp_crt_t, c_uint]
     gnutls_openpgp_crt_get_subkey_expiration_time.restype = time_t
+    gnutls_openpgp_crt_get_subkey_expiration_time.errmsg = "cannot get OpenPGP subkey expiration time"
 
     gnutls_openpgp_crt_get_subkey_fingerprint = libgnutls.gnutls_openpgp_crt_get_subkey_fingerprint
     gnutls_openpgp_crt_get_subkey_fingerprint.argtypes = [gnutls_openpgp_crt_t, c_uint, c_void_p, POINTER(size_t)]
@@ -1815,10 +1769,12 @@ else:
     gnutls_openpgp_privkey_get_subkey_creation_time = libgnutls.gnutls_openpgp_privkey_get_subkey_creation_time
     gnutls_openpgp_privkey_get_subkey_creation_time.argtypes = [gnutls_openpgp_privkey_t, c_uint]
     gnutls_openpgp_privkey_get_subkey_creation_time.restype = time_t
+    gnutls_openpgp_privkey_get_subkey_creation_time.errmsg = "cannot get OpenPGP subkey creation time"
 
     gnutls_openpgp_privkey_get_subkey_expiration_time = libgnutls.gnutls_openpgp_privkey_get_subkey_expiration_time
     gnutls_openpgp_privkey_get_subkey_expiration_time.argtypes = [gnutls_openpgp_privkey_t, c_uint]
     gnutls_openpgp_privkey_get_subkey_expiration_time.restype = time_t
+    gnutls_openpgp_privkey_get_subkey_expiration_time.errmsg = "cannot get OpenPGP subkey expiration time"
 
     gnutls_openpgp_privkey_get_subkey_fingerprint = libgnutls.gnutls_openpgp_privkey_get_subkey_fingerprint
     gnutls_openpgp_privkey_get_subkey_fingerprint.argtypes = [gnutls_openpgp_privkey_t, c_uint, c_void_p, POINTER(size_t)]
@@ -1864,6 +1820,70 @@ else:
     gnutls_openpgp_set_recv_key_function.argtypes = [gnutls_session_t, gnutls_openpgp_recv_key_func]
     gnutls_openpgp_set_recv_key_function.restype = None
 
+# The SRP related functions are not always present (some distributions do not compile SRP support into libgnutls)
+#
+
+try:
+    gnutls_srp_allocate_client_credentials = libgnutls.gnutls_srp_allocate_client_credentials
+except AttributeError:
+    pass
+else:
+    gnutls_srp_allocate_client_credentials = libgnutls.gnutls_srp_allocate_client_credentials
+    gnutls_srp_allocate_client_credentials.argtypes = [POINTER(gnutls_srp_client_credentials_t)]
+    gnutls_srp_allocate_client_credentials.restype = c_int
+
+    gnutls_srp_allocate_server_credentials = libgnutls.gnutls_srp_allocate_server_credentials
+    gnutls_srp_allocate_server_credentials.argtypes = [POINTER(gnutls_srp_server_credentials_t)]
+    gnutls_srp_allocate_server_credentials.restype = c_int
+
+    gnutls_srp_base64_decode = libgnutls.gnutls_srp_base64_decode
+    gnutls_srp_base64_decode.argtypes = [POINTER(gnutls_datum_t), c_char_p, POINTER(size_t)]
+    gnutls_srp_base64_decode.restype = c_int
+
+    gnutls_srp_base64_decode_alloc = libgnutls.gnutls_srp_base64_decode_alloc
+    gnutls_srp_base64_decode_alloc.argtypes = [POINTER(gnutls_datum_t), POINTER(gnutls_datum_t)]
+    gnutls_srp_base64_decode_alloc.restype = c_int
+
+    gnutls_srp_base64_encode = libgnutls.gnutls_srp_base64_encode
+    gnutls_srp_base64_encode.argtypes = [POINTER(gnutls_datum_t), c_char_p, POINTER(size_t)]
+    gnutls_srp_base64_encode.restype = c_int
+
+    gnutls_srp_base64_encode_alloc = libgnutls.gnutls_srp_base64_encode_alloc
+    gnutls_srp_base64_encode_alloc.argtypes = [POINTER(gnutls_datum_t), POINTER(gnutls_datum_t)]
+    gnutls_srp_base64_encode_alloc.restype = c_int
+
+    gnutls_srp_free_client_credentials = libgnutls.gnutls_srp_free_client_credentials
+    gnutls_srp_free_client_credentials.argtypes = [gnutls_srp_client_credentials_t]
+    gnutls_srp_free_client_credentials.restype = None
+
+    gnutls_srp_free_server_credentials = libgnutls.gnutls_srp_free_server_credentials
+    gnutls_srp_free_server_credentials.argtypes = [gnutls_srp_server_credentials_t]
+    gnutls_srp_free_server_credentials.restype = None
+
+    gnutls_srp_server_get_username = libgnutls.gnutls_srp_server_get_username
+    gnutls_srp_server_get_username.argtypes = [gnutls_session_t]
+    gnutls_srp_server_get_username.restype = c_char_p
+
+    gnutls_srp_set_client_credentials = libgnutls.gnutls_srp_set_client_credentials
+    gnutls_srp_set_client_credentials.argtypes = [gnutls_srp_client_credentials_t, c_char_p, c_char_p]
+    gnutls_srp_set_client_credentials.restype = c_int
+
+    gnutls_srp_set_client_credentials_function = libgnutls.gnutls_srp_set_client_credentials_function
+    gnutls_srp_set_client_credentials_function.argtypes = [gnutls_srp_client_credentials_t, gnutls_srp_client_credentials_function]
+    gnutls_srp_set_client_credentials_function.restype = None
+
+    gnutls_srp_set_server_credentials_file = libgnutls.gnutls_srp_set_server_credentials_file
+    gnutls_srp_set_server_credentials_file.argtypes = [gnutls_srp_server_credentials_t, c_char_p, c_char_p]
+    gnutls_srp_set_server_credentials_file.restype = c_int
+
+    gnutls_srp_set_server_credentials_function = libgnutls.gnutls_srp_set_server_credentials_function
+    gnutls_srp_set_server_credentials_function.argtypes = [gnutls_srp_server_credentials_t, gnutls_srp_server_credentials_function]
+    gnutls_srp_set_server_credentials_function.restype = None
+
+    gnutls_srp_verifier = libgnutls.gnutls_srp_verifier
+    gnutls_srp_verifier.argtypes = [c_char_p, c_char_p, POINTER(gnutls_datum_t), POINTER(gnutls_datum_t), POINTER(gnutls_datum_t), POINTER(gnutls_datum_t)]
+    gnutls_srp_verifier.restype = c_int
+
 
 __all__ = sorted(name for name, obj in sys.modules[__name__].__dict__.iteritems() if name.startswith('gnutls_') and hasattr(obj, 'restype'))
 
